#import "IMPLivePlayerComponent.h"
#import "AIMPSConf.h"
#import "IMPRoomChannelManager.h"
#import "FLNativeView.h"

@import AliInteractiveRoomBundle;
#import "IMPEvent.h"

#define PLAYER [IMPRoomChannelManager sharedInstance].room.livePlayer

@interface IMPLivePlayerComponent () <AIRBLivePlayerDelegate>
@property(copy, nonatomic) FlutterResult startCallback;
@end

@implementation IMPLivePlayerComponent

- (void)handleMethodCall:(FlutterMethodCall *)call result:(FlutterResult)result {
    NSString *apiName = call.method;
    NSLog([NSString stringWithFormat:@"iOS api invoked: %@", apiName]);

    if ([@"start" isEqualToString:apiName]) {
        [self start:call result:result];
    } else if ([@"pause" isEqualToString:apiName]) {
        [self pause:call result:result];
    } else if ([@"resume" isEqualToString:apiName]) {
        [self resume:call result:result];
    } else if ([@"stop" isEqualToString:apiName]) {
        [self stop:call result:result];
    } else if ([@"refresh" isEqualToString:apiName]) {
        [self refresh:call result:result];
    } else {
        result(FlutterMethodNotImplemented);
    }
}
- (void)start:(FlutterMethodCall *)call result:(FlutterResult)result {
    dispatch_async(dispatch_get_main_queue(), ^{
        NSLog(@"AAA addSubView PLAYER.playerView");

        UIView *container = [FLNativeViewFactory sharedInstance].view.view;
        [container addSubview:PLAYER.playerView];
    });
    PLAYER.delegate = self;

    if (call.arguments[@"showMode"]) {
        if ([call.arguments[@"showMode"] isEqualToString:@"fill"]) {
            PLAYER.contentMode = AIRBVideoViewContentModeFill;
        } else if ([call.arguments[@"showMode"] isEqualToString:@"aspect_fill"]) {
            PLAYER.contentMode = AIRBVideoViewContentModeAspectFill;
        } else if ([call.arguments[@"showMode"] isEqualToString:@"aspect_fit"]) {
            PLAYER.contentMode = AIRBVideoViewContentModeAspectFit;
        }
    }
    if (call.arguments[@"lowDelay"]) {
        PLAYER.lowDelay = [call.arguments[@"lowDelay"] boolValue];
    }

    _startCallback = result;
    [PLAYER start];
}

- (void)pause:(FlutterMethodCall *)call result:(FlutterResult)result {
    [PLAYER pause];
    [AIMPSUtils callOK:result];
}

- (void)resume:(FlutterMethodCall *)call result:(FlutterResult)result {
    [PLAYER resume];
    [AIMPSUtils callOK:result];
}

- (void)stop:(FlutterMethodCall *)call result:(FlutterResult)result {
    [PLAYER stop];
    [AIMPSUtils callOK:result];
}

- (void)refresh:(FlutterMethodCall *)call result:(FlutterResult)result {
    [PLAYER refresh];
    [AIMPSUtils callOK:result];
}

#pragma mark --AIRBLivePlayerDelegate

- (void)onAIRBLivePlayerEvent:(AIRBLivePlayerEvent)event info:(NSDictionary *)info {
    switch (event) {
        case AIRBLivePlayerEventStarted: {
            [IMPEvent fireEvent:@"liveFirstFrameRendered"];
            if (_startCallback) {
                [AIMPSUtils callOK:_startCallback];
                _startCallback = nil;
            }
        }
            break;
        case AIRBLivePlayerEventStartLoading: {
            [IMPEvent fireEvent:@"liveLoadingStarted"];
        }
            break;
        case AIRBLivePlayerEventEndLoading: {
            [IMPEvent fireEvent:@"liveLoadingStopped"];
        }
            break;
        case AIRBLivePlayerEventLiveEnded: {
            [IMPEvent fireEvent:@"liveStopped"];
        }
            break;
        case AIRBLivePlayerEventLiveNotExist: {
            [IMPEvent fireEvent:@"liveNotExist"];
        }
            break;;
        case AIRBLivePlayerEventLiveNotStarted: {
            [IMPEvent fireEvent:@"liveNotStarted"];
        }
            break;
        default:
            break;
    }
}

- (void)onAIRBLivePlayerErrorWithCode:(AIRBErrorCode)code message:(NSString *)msg {
    if (_startCallback) {
        [AIMPSUtils callErr:_startCallback msg:msg];
        _startCallback = nil;
    }
}
@end
