#import <AlivcLivePusher/AlivcLivePusher.h>
#import <AliInteractiveRoomBundle/AliInteractiveRoomBundle_Umbrella.h>
#import "IMPLivePusherComponent.h"
#import "IMPRoomChannelManager.h"

#import "AIMPSConf.h"
#import "FLNativeView.h"

@import AliInteractiveRoomBundle;
@import AlivcLivePusher;
#import "IMPEvent.h"

#define PUSHER [IMPRoomChannelManager sharedInstance].room.livePusher

@interface IMPLivePusherComponent () <AIRBLivePusherDelegate>
@property(copy, nonatomic) FlutterResult startPreviewCallback;
@property(copy, nonatomic) FlutterResult startLiveCallback;
@property(copy, nonatomic) FlutterResult stopLiveCallback;
@end

@implementation IMPLivePusherComponent

bool isPreviewed = false;
bool isBeautyPanelShown = false;

- (void)handleMethodCall:(FlutterMethodCall *)call result:(FlutterResult)result {
    NSString *apiName = call.method;
    NSLog([NSString stringWithFormat:@"iOS api invoked: %@", apiName]);

    if ([@"startPreview" isEqualToString:apiName]) {
        [self startPreview:call result:result];
    } else if ([@"toggleBeautyPanel" isEqualToString:apiName]) {
        [self toggleBeautyPanel:call result:result];
    } else if ([@"startLive" isEqualToString:apiName]) {
        [self startLive:call result:result];
    } else if ([@"setPreviewMode" isEqualToString:apiName]) {
        [self setPreviewMode:call result:result];
    } else if ([@"switchCamera" isEqualToString:apiName]) {
        [self switchCamera:call result:result];
    } else if ([@"setPreviewMirror" isEqualToString:apiName]) {
        [self setPreviewMirror:call result:result];
    } else if ([@"setPushMirror" isEqualToString:apiName]) {
        [self setPushMirror:call result:result];
    } else if ([@"stopLive" isEqualToString:apiName]) {
        [self stopLive:call result:result];
    } else {
        result(FlutterMethodNotImplemented);
    }
}

- (void)startPreview:(FlutterMethodCall *)call result:(FlutterResult)result {
    _startPreviewCallback = result;

    if (PUSHER == nil) {
        NSString *msg = @"startPreview fail: pusher nil";
        NSLog(msg);
        [AIMPSUtils callErr:result msg:msg];
        return;
    }
    PUSHER.delegate = self;

    NSDictionary *options = @{
            @"resolution": @"VALUE_720P",
    };
    AlivcLivePushConfig *config = [[AlivcLivePushConfig alloc] init];
    config.qualityMode = AlivcLivePushQualityModeCustom;
    config.enableAutoBitrate = NO;
    config.enableAutoResolution = NO;
    config.previewDisplayMode = ALIVC_LIVE_PUSHER_PREVIEW_ASPECT_FILL;
    config.orientation = AlivcLivePushOrientationPortrait;

    if (options[@"resolution"]) {
        NSLog([NSString stringWithFormat:@"AAA startPreview param: resolution=%@", options[@"resolution"]]);
        if ([options[@"resolution"] isEqualToString:@"VALUE_480P"]) {
            config.resolution = AlivcLivePushResolution480P;
        } else if ([options[@"resolution"] isEqualToString:@"VALUE_540P"]) {
            config.resolution = AlivcLivePushResolution540P;
        } else if ([options[@"resolution"] isEqualToString:@"VALUE_720P"]) {
            config.resolution = AlivcLivePushResolution720P;
        } else if ([options[@"resolution"] isEqualToString:@"VALUE_1080P"]) {
            //            config.resolution = AlivcLivePushResolution1080P;
        }
    }

    if (options[@"videoBitrate"]) {
        config.targetVideoBitrate = [options[@"videoBitrate"] intValue];
        config.minVideoBitrate = [options[@"videoBitrate"] intValue];
        config.initialVideoBitrate = [options[@"videoBitrate"] intValue];
    }

    if (options[@"showMode"]) {
        //        config.previewDisplayMode =
    }

    AIRBLivePusherOptions *option = [[AIRBLivePusherOptions alloc] init];
    option.mediaStreamingOptions.alivcLivePushConfig = config;

    [PUSHER startLocalPreviewWithOptions:option];
    [PUSHER setContentMode:AIRBVideoViewContentModeAspectFill];

    dispatch_async(dispatch_get_main_queue(), ^{
        NSLog(@"addSubView");

        UIView *container = [FLNativeViewFactory sharedInstance].view.view;
        [container addSubview:PUSHER.pusherView];
        isPreviewed = true;
    });
}

- (void)toggleBeautyPanel:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (!isPreviewed) {
        NSString *msg = @"toggleBeautyPanel failed: ensure startPreview success first";
        NSLog(msg);
        [AIMPSUtils callErr:result msg:msg];
        return;
    } else if (!isBeautyPanelShown) {
        UIWindow *window = [UIApplication sharedApplication].keyWindow;
        UIViewController *windowRootVC = window.rootViewController;
        UIView *panel = PUSHER.faceBeautyConfigViewController.view;
        [panel setFrame:CGRectMake(0, windowRootVC.view.frame.size.height - 200, panel.frame.size.width, 200)];
        panel.layer.zPosition = MAXFLOAT;
        [windowRootVC.view addSubview:panel];
        isBeautyPanelShown = true;
        [AIMPSUtils callOK:result];
    } else {
        UIView *panel = PUSHER.faceBeautyConfigViewController.view;
        [panel removeFromSuperview];
        isBeautyPanelShown = false;
        [AIMPSUtils callOK:result];
    }
}

- (void)startLive:(FlutterMethodCall *)call result:(FlutterResult)result {
    _startLiveCallback = result;
    [PUSHER startLiveStreaming];
}

- (void)setPreviewMode:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (call.arguments[@"mode"]) {
        [PUSHER setContentMode:(AIRBVideoViewContentMode) [call.arguments[@"mode"] intValue]];
        [AIMPSUtils callOK:result];
    } else {
        [AIMPSUtils callErr:result msg:[AIMPSUtils wrapInvalidParam:@"mode"]];
    }
}

- (void)switchCamera:(FlutterMethodCall *)call result:(FlutterResult)result {
    [PUSHER toggleCamera];
    [AIMPSUtils callOK:result];
}

- (void)setPreviewMirror:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (call.arguments[@"mirror"]) {
        [PUSHER setPreviewMirror:[call.arguments[@"mirror"] boolValue]];
        [AIMPSUtils callOK:result];
    } else {
        [AIMPSUtils callErr:result msg:@"mirror"];
    }
}

- (void)setPushMirror:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (call.arguments[@"mirror"]) {
        [PUSHER setStreamingVideoMirror:[call.arguments[@"mirror"] boolValue]];
        [AIMPSUtils callOK:result];
    } else {
        [AIMPSUtils callErr:result msg:@"mirror"];
    }
}

- (void)stopLive:(FlutterMethodCall *)call result:(FlutterResult)result {
    _stopLiveCallback = result;
    [PUSHER stopLiveStreaming];
}

#pragma mark -AIRBLivePusherDelegate

- (void)onAIRBLivePuhserEvent:(AIRBLivePusherEvent)event info:(NSDictionary *)info {
//    NSLog([NSString stringWithFormat:@"AAA onAIRBLivePuhserEvent: event=%i, info=%@", event, info]);
    switch (event) {
        case AIRBLivePusherEventStreamStarted: {
            if (_startLiveCallback) {
                [AIMPSUtils callOK:_startLiveCallback];
                _startLiveCallback = nil;
            }
            [IMPEvent fireEvent:@"livePushStarted"];
        }
            break;
        case AIRBLivePusherEventStopped: {
            if (_stopLiveCallback) {
                [AIMPSUtils callOK:_stopLiveCallback];
                _stopLiveCallback = nil;
            }
            isPreviewed = false;
            [IMPEvent fireEvent:@"livePushStopped"];
        }
            break;
        case AIRBLivePusherEventPreviewStarted: {
            if (_startPreviewCallback) {
                [AIMPSUtils callOK:_startPreviewCallback];
                _startPreviewCallback = nil;
            }
            isPreviewed = true;
            [IMPEvent fireEvent:@"livePushPreviewStarted"];
        }
            break;
        case AIRBLivePusherEventNetworkPoored: {
            [IMPEvent fireEvent:@"livePushNetworkPoored"];
        }
            break;
        case AIRBLivePusherEventNetworkRecoveried: {
            [IMPEvent fireEvent:@"livePushNetworkRecoveried"];
        }
            break;
        case AIRBLivePusherEventNetworkConnectionLost: {
            [IMPEvent fireEvent:@"livePushConnectionLost"];
        }
            break;
        case AIRBLivePusherEventNetworkReconnectStart: {
            [IMPEvent fireEvent:@"liveReconnectStart"];
        }
            break;
        case AIRBLivePusherEventNetworkReconnectSuccess: {
            [IMPEvent fireEvent:@"liveReconnectSuccess"];
        }
            break;
        case AIRBLivePusherEventNetworkReconnectFailed: {
            [IMPEvent fireEvent:@"liveReconnectFail"];
        }
            break;
        default:
            break;
    }
}

- (void)onAIRBLivePusherError:(AIRBErrorCode)errorCode message:(NSString *)errorMessage {
//    [self fireEvent:@"error" params:@{
//            @"code" : @(errorCode),
//            @"msg" : errorMessage ?: @""
//    }];
}

@end
