#import <AliInteractiveRoomBundle/AliInteractiveRoomBundle_Umbrella.h>
#import <alicloud_impinteraction_sdk/IMPRoomChannelManager.h>
#import "IMPEvent.h"
#import "IMPRoomChannelModule.h"

#import "AIMPSConf.h"

@interface IMPRoomChannelModule ()

#define ROOM [IMPRoomChannelManager sharedInstance].room

@property(nonatomic, strong) NSString *roomId;

@property(strong, nonatomic) AIRBRoomBasicInfo *roomDetail;
@property(copy, nonatomic) FlutterResult enterRoomCallback;
@property(copy, nonatomic) FlutterResult leaveRoomCallback;
@end


@implementation IMPRoomChannelModule

#pragma mark -Properties

#pragma mark -Lifycycle

#pragma mark -Public

- (void)handleMethodCall:(FlutterMethodCall *)call result:(FlutterResult)result {
    NSString *apiName = call.method;
    NSLog([NSString stringWithFormat:@"iOS api invoked: %@", apiName]);

    if ([@"getRoomId" isEqualToString:apiName]) {
        [self getRoomId:call result:result];
    } else if ([@"isOwner" isEqualToString:apiName]) {
        [self isOwner:call result:result];
    } else if ([@"isAdmin" isEqualToString:apiName]) {
        [self isAdmin:call result:result];
    } else if ([@"setRoomId" isEqualToString:apiName]) {
        [self setRoomId:call result:result];
    } else if ([@"enterRoom" isEqualToString:apiName]) {
        [self enterRoom:call result:result];
    } else if ([@"leaveRoom" isEqualToString:apiName]) {
        [self leaveRoom:call result:result];
    } else if ([@"listUser" isEqualToString:apiName]) {
        [self listUser:call result:result];
    } else if ([@"getRoomDetail" isEqualToString:apiName]) {
        [self getRoomDetail:call result:result];
    } else if ([@"kickUser" isEqualToString:apiName]) {
        [self kickUser:call result:result];
    } else {
        result(FlutterMethodNotImplemented);
    }
}

- (void)getRoomId:(FlutterMethodCall *)call result:(FlutterResult)result {
    [AIMPSUtils callOK:result msg:[IMPRoomChannelManager sharedInstance].roomID];
}

- (void)isOwner:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (!_roomDetail) {
        __weak typeof(self) weakSelf = self;
        [ROOM getRoomDetail:^(AIRBRoomBasicInfo *_Nullable roomDetail) {
            weakSelf.roomDetail = roomDetail;
            [self invokeCallbackForIsOwner:result];
        }];
    } else {
        [self invokeCallbackForIsOwner:result];
    }
}

- (void)invokeCallbackForIsOwner:(FlutterResult)result {
    [AIMPSUtils callOK:result condition:[[AIMPSConf ins].userId isEqualToString:_roomDetail.ownerID]];
}

- (void)isAdmin:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (!_roomDetail) {
        __weak typeof(self) weakSelf = self;
        [ROOM getRoomDetail:^(AIRBRoomBasicInfo *_Nullable roomDetail) {
            weakSelf.roomDetail = roomDetail;
            [self invokeCallbackForIsAdmin:result detai:roomDetail];
        }];
    } else {
        [self invokeCallbackForIsAdmin:result detai:_roomDetail];
    }
}

- (void)invokeCallbackForIsAdmin:(FlutterResult)result detai:(AIRBRoomBasicInfo *_Nullable)roomDetail {
    NSArray<NSString *> *administers = roomDetail.administers;
    for (id object in administers) {
        if ([[AIMPSConf ins].userId isEqualToString:object]) {
            [AIMPSUtils callOK:result];
            return;
        }
    }
    [AIMPSUtils callOK:result condition:false];
}

- (void)setRoomId:(FlutterMethodCall *)call result:(FlutterResult)result {
    _roomId = call.arguments[@"roomId"];

    if (_roomId) {
        [[IMPRoomChannelManager sharedInstance] setupWithRoomID:_roomId];
        [IMPRoomChannelManager sharedInstance].room.delegate = [IMPRoomChannelManager sharedInstance];

        __weak typeof(self) weakSelf = self;
        [IMPRoomChannelManager sharedInstance].onRoomError = ^(AIRBErrorCode code, NSString *_Nonnull message) {
            [weakSelf p_handleRoomError:code message:message];
        };
        [IMPRoomChannelManager sharedInstance].onRoomEvent = ^(AIRBRoomChannelEvent event, NSDictionary *_Nonnull info) {
            [weakSelf p_handleRoomEvent:event info:info];
        };
        [IMPRoomChannelManager sharedInstance].onReceiveRoomMessage = ^(AIRBRoomChannelMessageType type, NSDictionary *_Nonnull info) {
            [weakSelf p_handleRoomMessage:type info:info];
        };

        [AIMPSUtils callOK:result];
    } else {
        [AIMPSUtils callErr:result msg:[AIMPSUtils wrapInvalidParam:@"roomId"]];
    }
}

- (void)enterRoom:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (call.arguments[@"nick"]) {
        NSString *nick = call.arguments[@"nick"];
        NSDictionary *extension = nil;
        if (call.arguments[@"extension"] && [call.arguments[@"extension"] isKindOfClass:[NSDictionary class]]) {
            extension = call.arguments[@"extension"];
        }
        _enterRoomCallback = result;
        [[IMPRoomChannelManager sharedInstance].room enterRoomWithUserNick:nick extension:extension];
    } else {
        [AIMPSUtils callErr:result msg:@"invalid nick."];
    }
}

- (void)leaveRoom:(FlutterMethodCall *)call result:(FlutterResult)result {
    _leaveRoomCallback = result;
    [ROOM leaveRoom];
}

- (void)listUser:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (call.arguments[@"pageSize"] && call.arguments[@"pageNum"]) {
        [ROOM getRoomUserListWithPageNum:[call.arguments[@"pageNum"] intValue] pageSize:[call.arguments[@"pageSize"] intValue] onSuccess:^(AIRBRoomChannelUserListResponse *_Nonnull response) {
            NSMutableDictionary *contentDictionary = [@{
                    @"total": @(response.totalCount),
                    @"hasMore": @(response.hasMore),
                    @"list": response.userList
            } mutableCopy];
            [AIMPSUtils take:result dic:contentDictionary];
        }                      onFailure:^(NSString *_Nonnull errorMessage) {
            [AIMPSUtils callErr:result msg:errorMessage];
        }];
    } else {
        [AIMPSUtils callErr:result msg:@"invalid pageNub=m or pageSize."];
    }
}

- (void)getRoomDetail:(FlutterMethodCall *)call result:(FlutterResult)result {
    __weak typeof(self) weakSelf = self;
    [ROOM getRoomDetail:^(AIRBRoomBasicInfo *_Nullable roomDetail) {
        weakSelf.roomDetail = roomDetail;
        NSMutableDictionary *contentDictionary = [[NSMutableDictionary alloc] init];
        [contentDictionary setValue:@{
                @"roomId": roomDetail.roomID ?: @"",
                @"title": roomDetail.title ?: @"",
                @"notice": roomDetail.notice ?: @"",
                @"ownerId": roomDetail.ownerID ?: @"",
                @"uv": [NSString stringWithFormat:@"%d", roomDetail.uv],
                @"pv": [NSString stringWithFormat:@"%d", roomDetail.pv],
                @"onlineCount": [NSString stringWithFormat:@"%d", roomDetail.onlineCount],
                @"extension": roomDetail.extension,
                @"adminIdList": roomDetail.administers,
        }                    forKey:@"roomInfo"];
        [AIMPSUtils take:result dic:contentDictionary];
    }];
}

- (void)kickUser:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (call.arguments[@"userId"]) {
        int32_t kickedSeconds = 0;
        if (call.arguments[@"kickedSeconds"]) {
            kickedSeconds = [call.arguments[@"kickedSeconds"] intValue];
        }
        [ROOM kickRoomUserWithUserID:call.arguments[@"userId"] kickedSeconds:kickedSeconds onSuccess:^{
            [AIMPSUtils callOK:result];
        }                  onFailure:^(NSString *_Nonnull errorMessage) {
            [AIMPSUtils callErr:result msg:errorMessage ?: @""];
        }];
    } else {
        [AIMPSUtils callErr:result msg:@"invalid userId"];
    }
}

#pragma mark -Private

- (void)p_handleRoomEvent:(AIRBRoomChannelEvent)event info:(NSDictionary *)info {
    NSLog(@"p_handleRoomEvent %i", event);
    switch (event) {
        case AIRBRoomChannelEventEntered: {
            if (_enterRoomCallback) {
                [AIMPSUtils callOK:_enterRoomCallback];
                _enterRoomCallback = nil;
            }
        }
            break;
        case AIRBRoomChannelEventLeft: {
            if (_leaveRoomCallback) {
                [AIMPSUtils callOK:_leaveRoomCallback];
                _leaveRoomCallback = nil;
            }
        }
            break;
        case AIRBRoomChannelEventRoomInfoGotten: {

        }
            break;

        default:
            break;
    }
}

- (void)p_handleRoomError:(AIRBErrorCode)code message:(NSString *)message {
    switch (code) {
        case AIRBRoomChannelFailedToEnterRoom: {
            if (_enterRoomCallback) {
                [AIMPSUtils callErr:_enterRoomCallback msg:message];
                _enterRoomCallback = nil;
            }
        }
            break;
        case AIRBRoomChannelFailedToLeaveRoom: {
            if (_leaveRoomCallback) {
                [AIMPSUtils callErr:_leaveRoomCallback msg:message];
                _leaveRoomCallback = nil;
            }
        }

        default:
            break;
    }
}

- (void)p_handleRoomMessage:(AIRBRoomChannelMessageType)type info:(NSDictionary *)info {
    NSString *command = nil;
    switch (type) {
        case AIRBRoomChannelMessageTypeRoomTitleUpdated: {
            command = @"roomTitleUpdated";
        }
            break;
        case AIRBRoomChannelMessageTypeRoomMembersInOut: {
            if ([[info valueForKey:@"enter"] boolValue]) {
                command = @"enterRoom";
            } else {
                command = @"leaveRoom";
            }
        }
            break;
        case AIRBRoomChannelMessageTypeRoomNoticeUpdated: {
            command = @"roomNoticeUpdated";
        }
            break;
        case AIRBRoomChannelMessageTypeRoomNoticeUpdatedNew: {
            command = @"roomNoticeUpdated";
        }
            break;
        case AIRBRoomChannelMessageTypeRoomExtensionUpdated: {
            command = @"roomExtensionUpdated";
        }
            break;
        case AIRBRoomChannelMessageTypeRoomOneUserKickedOut: {
            command = @"kickedFromRoom";
        }
            break;
        default:
            break;
    }
    if (command) {
        [IMPEvent fireEvent:command params:info];
    }
}

@end
