#import "IMPRoomEngineModule.h"

#import "AIMPSConf.h"
#import "FLNativeView.h"
#import <AlivcLivePusher/AlivcLivePusher.h>
#import "IMPEvent.h"

typedef void (^onTokenGottenCallback)(AIRBRoomEngineAuthToken *token);

@interface IMPRoomEngineModule () <AIRBRoomEngineDelegate>
@property(strong, nonatomic) AIRBRoomEngineConfig *engineConfig;

@property(nonatomic, strong) NSString *userId;
@property(nonatomic, strong) NSString *appId;
@property(nonatomic, strong) NSString *appKey;
@property(nonatomic, strong) NSString *serverHost;
@property(nonatomic, strong) NSString *serverSecret;
@property(nonatomic, strong) NSString *deviceId;

@property(copy, nonatomic) FlutterResult initCallback;
@property(copy, nonatomic) FlutterResult loginCallback;
@property(copy, nonatomic) onTokenGottenCallback tokenGottenCallback;
@end

@implementation IMPRoomEngineModule

- (void)handleMethodCall:(FlutterMethodCall *)call result:(FlutterResult)result {
    NSString *apiName = call.method;
    NSLog([NSString stringWithFormat:@"iOS api invoked: %@", apiName]);

    if ([@"init" isEqualToString:apiName]) {
        [self init:call result:result];
    } else if ([@"login" isEqualToString:apiName]) {
        [self login:call result:result];
    } else if ([@"setLoginToken" isEqualToString:apiName]) {
        [self setLoginToken:call result:result];
    } else if ([@"logout" isEqualToString:apiName]) {
        [self logout:call result:result];
    } else if ([@"isLogined" isEqualToString:apiName]) {
        [self isLogined:call result:result];
    } else {
        result(FlutterMethodNotImplemented);
    }
}

- (void)init:(FlutterMethodCall *)call result:(FlutterResult)result {
    AIRBRoomEngineConfig *ec = [[AIRBRoomEngineConfig alloc] init];
    [AIMPSConf ins].userId = _userId = call.arguments[@"userId"];
    ec.appID = [AIMPSConf ins].appId = _appId = call.arguments[@"appId"];
    ec.appKey = [AIMPSConf ins].appKey = _appKey = call.arguments[@"appKey4iOS"];
    [AIMPSConf ins].serverHost = _serverHost = call.arguments[@"serverHost"];
    [AIMPSConf ins].serverSecret = _serverSecret = call.arguments[@"serverSecret"];
    ec.deviceID = [AIMPSConf ins].deviceId = _deviceId = call.arguments[@"deviceId"] ? call.arguments[@"deviceId"] : [[UIDevice currentDevice] identifierForVendor].UUIDString;

    _initCallback = result;
    [AIRBRoomEngine sharedInstance].delegate = self;
    [[AIRBRoomEngine sharedInstance] globalInitOnceWithConfig:ec];
}

- (void)login:(FlutterMethodCall *)call result:(FlutterResult)result {
    NSLog([NSString stringWithFormat:@"AAA iOS login: %@", _userId]);
    _loginCallback = result;
    [[AIRBRoomEngine sharedInstance] loginWithUserID:_userId];
}

- (void)setLoginToken:(FlutterMethodCall *)call result:(FlutterResult)result {
    AIRBRoomEngineAuthToken *token = [[AIRBRoomEngineAuthToken alloc] init];
    token.refreshToken = call.arguments[@"refreshToken"];
    token.accessToken = call.arguments[@"accessToken"];
//    NSLog([NSString stringWithFormat:@"AAA iOS token= refreshToken: %@, accessToken: %@", token.refreshToken, token.accessToken]);
    _tokenGottenCallback(token);
    _tokenGottenCallback = nil;

    [AIMPSUtils callOK:result];
}

- (void)logout:(FlutterMethodCall *)call result:(FlutterResult)result {
    [[AIRBRoomEngine sharedInstance] logoutOnSuccess:^{
        [AIMPSUtils callOK:result];
    }                                      onFailure:^(NSString *_Nonnull errorMessage) {
        [AIMPSUtils callErr:result msg:errorMessage];
    }];
}

- (void)isLogined:(FlutterMethodCall *)call result:(FlutterResult)result {
    [AIMPSUtils callOK:result condition:[[AIRBRoomEngine sharedInstance] isLogined:_userId]];
}

#pragma mark-AIRBRoomEngineDelegate

- (void)onAIRBRoomEngineRequestToken:(void (^)(AIRBRoomEngineAuthToken *_Nonnull))onTokenGotten {
    _tokenGottenCallback = onTokenGotten;
    [IMPEvent fireEvent:@"onRequestToken"];
}

- (void)onAIRBRoomEngineEvent:(AIRBRoomEngineEvent)event info:(NSDictionary *)info {
    NSLog(@"onAIRBRoomEngineEvent %i", event);

    switch (event) {
        case AIRBRoomEngineEventEngineStarted: {
            if (_initCallback) {
                [AIMPSUtils callOK:_initCallback];
                _initCallback = nil;
            }
            break;
        }
        case AIRBRoomEngineEventEngineLogined: {
            if (_loginCallback) {
                [AIMPSUtils callOK:_loginCallback];
                _loginCallback = nil;
            }
            break;
        }
        case AIRBRoomEngineEventEngineKickedOut: {
            [IMPEvent fireEvent:@"onKicked"];
            break;
        }
        default:
            break;
    }
}

- (void)onAIRBRoomEngineErrorWithCode:(AIRBErrorCode)code errorMessage:(NSString *)msg {
    NSLog([NSString stringWithFormat:@"onAIRBRoomEngineErrorWithCode code:%i msg:%@", code, msg]);
    switch (code) {
        case AIRBEngineFailedToStart:
        case AIRBEngineFailedToCreateDPSManager: {
            if (_initCallback) {
                [AIMPSUtils callErr:_initCallback msg:msg];
                _initCallback = nil;
            }
        }
            break;
        case AIRBEngineFailedToLogin: {
            if (_loginCallback) {
                [AIMPSUtils callErr:_loginCallback];
                _loginCallback = nil;
            }
        }
            break;
        default:
            break;
    }
}

@end
